@@ -23,11 +23,11 @@
 
 /**
  * SECTION:intset
- * @title: TpIntSet
+ * @title: TpIntset
  * @short_description: a set of unsigned integers
  * @see_also: #TpHandleSet
  *
- * A #TpIntSet is a set of unsigned integers, implemented as a
+ * A #TpIntset is a set of unsigned integers, implemented as a
  * dynamically-allocated sparse bitfield.
  */
 
@@ -59,7 +59,7 @@ G_STATIC_ASSERT (sizeof (gpointer) >= sizeof (gsize));
 /**
  * TP_TYPE_INTSET:
  *
- * The boxed type of a #TpIntSet.
+ * The boxed type of a #TpIntset.
  *
  * Since: 0.11.3
  */
@@ -71,7 +71,12 @@ tp_intset_get_type (void)
 
   if (G_UNLIKELY (type == 0))
     {
-      type = g_boxed_type_register_static (g_intern_static_string ("TpIntSet"),
+      /* The "TpIntSet" type has to be registered for backwards compatibility.
+       * The canonical name of the type is now "TpIntset"; see fdo#30134. */
+      g_boxed_type_register_static (g_intern_static_string ("TpIntSet"),
+          (GBoxedCopyFunc) tp_intset_copy,
+          (GBoxedFreeFunc) tp_intset_destroy);
+      type = g_boxed_type_register_static (g_intern_static_string ("TpIntset"),
           (GBoxedCopyFunc) tp_intset_copy,
           (GBoxedFreeFunc) tp_intset_destroy);
     }
@@ -89,7 +94,7 @@ tp_intset_get_type (void)
 /* (typedef, see header) */
 
 /**
- * TpIntSetIter:
+ * TpIntsetIter:
  * @set: The set iterated over.
  * @element: Must be (guint)(-1) before iteration starts. Set to the next
  *  element in the set by tp_intset_iter_next(); undefined after
@@ -98,9 +103,11 @@ tp_intset_get_type (void)
  * A structure representing iteration over a set of integers. Must be
  * initialized with either TP_INTSET_ITER_INIT() or tp_intset_iter_init().
  *
- * Since 0.11.6, consider using #TpIntSetFastIter if iteration in
+ * Since 0.11.6, consider using #TpIntsetFastIter if iteration in
  * numerical order is not required.
  *
+ * Before 0.11.16, this type was called <type>TpIntSetIter</type>,
+ * which is now a backwards compatibility typedef.
  */
 /* (public, see header) */
 
@@ -108,18 +115,18 @@ tp_intset_get_type (void)
  * TP_INTSET_ITER_INIT:
  * @set: A set of integers
  *
- * A suitable static initializer for a #TpIntSetIter, to be used as follows:
+ * A suitable static initializer for a #TpIntsetIter, to be used as follows:
  *
  * <informalexample><programlisting>
  * void
- * do_something (const TpIntSet *intset)
+ * do_something (const TpIntset *intset)
  * {
- *   TpIntSetIter iter = TP_INTSET_ITER_INIT (intset);
+ *   TpIntsetIter iter = TP_INTSET_ITER_INIT (intset);
  *   /<!-- -->* ... do something with iter ... *<!-- -->/
  * }
  * </programlisting></informalexample>
  *
- * Since 0.11.6, consider using #TpIntSetFastIter if iteration in
+ * Since 0.11.6, consider using #TpIntsetFastIter if iteration in
  * numerical order is not required.
  *
  */
@@ -144,12 +151,15 @@ tp_intset_get_type (void)
 /* (inline, see header) */
 
 /**
- * TpIntSet:
+ * TpIntset:
  *
  * Opaque type representing a set of unsigned integers.
+ *
+ * Before 0.11.16, this type was called <type>TpIntSet</type>, which is
+ * now a backwards compatibility typedef.
  */
 
-struct _TpIntSet
+struct _TpIntset
 {
   /* HIGH_PART(n) => bitfield where bit LOW_PART(n) is set if n is present.
    *
@@ -166,11 +176,11 @@ struct _TpIntSet
  *
  * We could use g_bit_nth_msf (value, BITFIELD_BITS) instead of LOW_MASK if we
  * wanted to get largest_ever exactly right, but we just need something
- * reasonable to make TpIntSetIter terminate early, and carrying on for up to
+ * reasonable to make TpIntsetIter terminate early, and carrying on for up to
  * BITFIELD_BITS extra iterations isn't a problem.
  */
 static inline void
-intset_update_largest_ever (TpIntSet *set,
+intset_update_largest_ever (TpIntset *set,
     gpointer key)
 {
   guint upper_bound = GPOINTER_TO_UINT (key) | LOW_MASK;
@@ -188,7 +198,7 @@ intset_update_largest_ever (TpIntSet *set,
  *
  * Returns: a new, empty integer set to be destroyed with tp_intset_destroy()
  */
-TpIntSet *
+TpIntset *
 tp_intset_sized_new (guint size G_GNUC_UNUSED)
 {
   return tp_intset_new ();
@@ -201,10 +211,10 @@ tp_intset_sized_new (guint size G_GNUC_UNUSED)
  *
  * Returns: a new, empty integer set to be destroyed with tp_intset_destroy()
  */
-TpIntSet *
+TpIntset *
 tp_intset_new ()
 {
-  TpIntSet *set = g_slice_new (TpIntSet);
+  TpIntset *set = g_slice_new (TpIntset);
 
   set->table = g_hash_table_new (NULL, NULL);
   set->largest_ever = 0;
@@ -220,12 +230,12 @@ tp_intset_new ()
  * Returns: a new integer set containing @element, to be destroyed with
  * tp_intset_destroy()
  *
- * @since 0.7.26
+ * Since: 0.7.26
  */
-TpIntSet *
+TpIntset *
 tp_intset_new_containing (guint element)
 {
-  TpIntSet *ret = tp_intset_new ();
+  TpIntset *ret = tp_intset_new ();
 
   tp_intset_add (ret, element);
 
@@ -239,12 +249,12 @@ tp_intset_new_containing (guint element)
  * Free all memory used by the set.
  */
 void
-tp_intset_destroy (TpIntSet *set)
+tp_intset_destroy (TpIntset *set)
 {
   g_return_if_fail (set != NULL);
 
   g_hash_table_destroy (set->table);
-  g_slice_free (TpIntSet, set);
+  g_slice_free (TpIntset, set);
 }
 
 /**
@@ -254,7 +264,7 @@ tp_intset_destroy (TpIntSet *set)
  * Unset every integer in the set.
  */
 void
-tp_intset_clear (TpIntSet *set)
+tp_intset_clear (TpIntset *set)
 {
   g_return_if_fail (set != NULL);
 
@@ -266,10 +276,10 @@ tp_intset_clear (TpIntSet *set)
  * @set: set
  * @element: integer to add
  *
- * Add an integer into a TpIntSet.
+ * Add an integer into a TpIntset.
  */
 void
-tp_intset_add (TpIntSet *set,
+tp_intset_add (TpIntset *set,
     guint element)
 {
   gpointer key = GSIZE_TO_POINTER ((gsize) HIGH_PART (element));
@@ -293,12 +303,12 @@ tp_intset_add (TpIntSet *set,
  * @set: set
  * @element: integer to add
  *
- * Remove an integer from a TpIntSet
+ * Remove an integer from a TpIntset
  *
  * Returns: %TRUE if @element was previously in @set
  */
 gboolean
-tp_intset_remove (TpIntSet *set,
+tp_intset_remove (TpIntset *set,
     guint element)
 {
   gpointer key = GSIZE_TO_POINTER ((gsize) HIGH_PART (element));
@@ -324,7 +334,7 @@ tp_intset_remove (TpIntSet *set,
 }
 
 static inline gboolean
-_tp_intset_is_member (const TpIntSet *set,
+_tp_intset_is_member (const TpIntset *set,
     guint element)
 {
   gpointer key = GSIZE_TO_POINTER ((gsize) HIGH_PART (element));
@@ -345,7 +355,7 @@ _tp_intset_is_member (const TpIntSet *set,
  * Returns: %TRUE if @element is in @set
  */
 gboolean
-tp_intset_is_member (const TpIntSet *set, guint element)
+tp_intset_is_member (const TpIntset *set, guint element)
 {
   g_return_val_if_fail (set != NULL, FALSE);
 
@@ -355,14 +365,14 @@ tp_intset_is_member (const TpIntSet *set, guint element)
 /**
  * tp_intset_foreach:
  * @set: set
- * @func: @TpIntFunc to use to iterate the set
+ * @func: (scope call): @TpIntFunc to use to iterate the set
  * @userdata: user data to pass to each call of @func
  *
  * Call @func(element, @userdata) for each element of @set, in order.
  */
 
 void
-tp_intset_foreach (const TpIntSet *set,
+tp_intset_foreach (const TpIntset *set,
     TpIntFunc func,
     gpointer userdata)
 {
@@ -404,11 +414,11 @@ addint (guint i, gpointer data)
  *
  * <!--Returns: says it all-->
  *
- * Returns: (element-type uint): a GArray of guint (which must be freed by the caller) containing
- * the same integers as @set.
+ * Returns: (element-type uint) (transfer full): a GArray of guint (which must
+ *  be freed by the caller) containing the same integers as @set.
  */
 GArray *
-tp_intset_to_array (const TpIntSet *set)
+tp_intset_to_array (const TpIntset *set)
 {
   GArray *array;
 
@@ -431,10 +441,10 @@ tp_intset_to_array (const TpIntSet *set)
  * Returns: A set containing the same integers as @array.
  */
 
-TpIntSet *
+TpIntset *
 tp_intset_from_array (const GArray *array)
 {
-  TpIntSet *set;
+  TpIntset *set;
   guint i;
 
   g_return_val_if_fail (array != NULL, NULL);
@@ -469,7 +479,7 @@ count_bits32 (guint32 n)
  */
 
 guint
-tp_intset_size (const TpIntSet *set)
+tp_intset_size (const TpIntset *set)
 {
   guint count = 0;
   gpointer entry;
@@ -499,7 +509,7 @@ tp_intset_size (const TpIntSet *set)
  * Since: 0.11.6
  */
 gboolean
-tp_intset_is_empty (const TpIntSet *set)
+tp_intset_is_empty (const TpIntset *set)
 {
   g_return_val_if_fail (set != NULL, TRUE);
   return (g_hash_table_size (set->table) == 0);
@@ -516,8 +526,8 @@ tp_intset_is_empty (const TpIntSet *set)
  */
 
 gboolean
-tp_intset_is_equal (const TpIntSet *left,
-    const TpIntSet *right)
+tp_intset_is_equal (const TpIntset *left,
+    const TpIntset *right)
 {
   gpointer key, value;
   GHashTableIter iter;
@@ -552,12 +562,12 @@ tp_intset_is_equal (const TpIntSet *left,
  * tp_intset_destroy() by the caller
  */
 
-TpIntSet *
-tp_intset_copy (const TpIntSet *orig)
+TpIntset *
+tp_intset_copy (const TpIntset *orig)
 {
   gpointer key, value;
   GHashTableIter iter;
-  TpIntSet *ret;
+  TpIntset *ret;
 
   g_return_val_if_fail (orig != NULL, NULL);
 
@@ -587,12 +597,12 @@ tp_intset_copy (const TpIntSet *orig)
  * tp_intset_destroy() by the caller
  */
 
-TpIntSet *
-tp_intset_intersection (const TpIntSet *left, const TpIntSet *right)
+TpIntset *
+tp_intset_intersection (const TpIntset *left, const TpIntset *right)
 {
   gpointer key, value;
   GHashTableIter iter;
-  TpIntSet *ret;
+  TpIntset *ret;
 
   ret = tp_intset_new ();
 
@@ -614,7 +624,6 @@ tp_intset_intersection (const TpIntSet *left, const TpIntSet *right)
   return ret;
 }
 
-
 /**
  * tp_intset_union:
  * @left: The left operand
@@ -627,30 +636,46 @@ tp_intset_intersection (const TpIntSet *left, const TpIntSet *right)
  * tp_intset_destroy() by the caller
  */
 
-TpIntSet *
-tp_intset_union (const TpIntSet *left, const TpIntSet *right)
+TpIntset *
+tp_intset_union (const TpIntset *left, const TpIntset *right)
+{
+  TpIntset *ret;
+
+  ret = tp_intset_copy (left);
+  tp_intset_union_update (ret, right);
+
+  return ret;
+}
+
+/**
+ * tp_intset_union_update:
+ * @self: the set to change
+ * @other: members to add
+ *
+ * Add each integer in @other to @self, analogous to the bitwise operation
+ * self |= other.
+ *
+ * Since: 0.13.10
+ */
+void
+tp_intset_union_update (TpIntset *self,
+    const TpIntset *other)
 {
   gpointer key, value;
   GHashTableIter iter;
-  TpIntSet *ret;
 
-  ret = tp_intset_copy (left);
-
-  g_hash_table_iter_init (&iter, (GHashTable *) right->table);
+  g_hash_table_iter_init (&iter, (GHashTable *) other->table);
 
   while (g_hash_table_iter_next (&iter, &key, &value))
     {
       gsize v = GPOINTER_TO_SIZE (value);
 
-      intset_update_largest_ever (ret, key);
-      v |= GPOINTER_TO_SIZE (g_hash_table_lookup (ret->table, key));
-      g_hash_table_insert (ret->table, key, GSIZE_TO_POINTER (v));
+      intset_update_largest_ever (self, key);
+      v |= GPOINTER_TO_SIZE (g_hash_table_lookup (self->table, key));
+      g_hash_table_insert (self->table, key, GSIZE_TO_POINTER (v));
     }
-
-  return ret;
 }
 
-
 /**
  * tp_intset_difference:
  * @left: The left operand
@@ -663,36 +688,52 @@ tp_intset_union (const TpIntSet *left, const TpIntSet *right)
  * tp_intset_destroy() by the caller
  */
 
-TpIntSet *
-tp_intset_difference (const TpIntSet *left, const TpIntSet *right)
+TpIntset *
+tp_intset_difference (const TpIntset *left, const TpIntset *right)
 {
-  TpIntSet *ret;
-  gpointer key, value;
-  GHashTableIter iter;
+  TpIntset *ret;
 
   g_return_val_if_fail (left != NULL, NULL);
   g_return_val_if_fail (right != NULL, NULL);
 
   ret = tp_intset_copy (left);
-
-  g_hash_table_iter_init (&iter, (GHashTable *) right->table);
-
-  while (g_hash_table_iter_next (&iter, &key, &value))
-    {
-      gsize v = GPOINTER_TO_SIZE (value);
-      v = (GPOINTER_TO_SIZE (g_hash_table_lookup (ret->table, key))) & ~v;
-
-      /* No need to update largest_ever here - we're only deleting members. */
-
-      if (v == 0)
-        g_hash_table_remove (ret->table, key);
-      else
-        g_hash_table_insert (ret->table, key, GSIZE_TO_POINTER (v));
-    }
+  tp_intset_difference_update (ret, right);
 
   return ret;
 }
 
+/**
+ * tp_intset_difference_update:
+ * @self: the set to change
+ * @other: members to remove
+ *
+ * Remove each integer in @other from @self, analogous to the bitwise
+ * operation self &= (~other).
+ *
+ * Since: 0.13.10
+ */
+void
+tp_intset_difference_update (TpIntset *self,
+    const TpIntset *other)
+{
+  gpointer key, value;
+  GHashTableIter iter;
+
+  g_hash_table_iter_init (&iter, (GHashTable *) other->table);
+
+  while (g_hash_table_iter_next (&iter, &key, &value))
+    {
+      gsize v = GPOINTER_TO_SIZE (value);
+      v = (GPOINTER_TO_SIZE (g_hash_table_lookup (self->table, key))) & ~v;
+
+      /* No need to update largest_ever here - we're only deleting members. */
+
+      if (v == 0)
+        g_hash_table_remove (self->table, key);
+      else
+        g_hash_table_insert (self->table, key, GSIZE_TO_POINTER (v));
+    }
+}
 
 /**
  * tp_intset_symmetric_difference:
@@ -706,10 +747,10 @@ tp_intset_difference (const TpIntSet *left, const TpIntSet *right)
  * with tp_intset_destroy() by the caller
  */
 
-TpIntSet *
-tp_intset_symmetric_difference (const TpIntSet *left, const TpIntSet *right)
+TpIntset *
+tp_intset_symmetric_difference (const TpIntset *left, const TpIntset *right)
 {
-  TpIntSet *ret;
+  TpIntset *ret;
   gpointer key, value;
   GHashTableIter iter;
 
@@ -757,7 +798,7 @@ _dump_foreach (guint i, gpointer data)
  * numbers in @set in a human-readable format
  */
 gchar *
-tp_intset_dump (const TpIntSet *set)
+tp_intset_dump (const TpIntset *set)
 {
   GString *tmp = g_string_new ("");
 
@@ -775,20 +816,20 @@ tp_intset_dump (const TpIntSet *set)
  * Usage:
  *
  * <informalexample><programlisting>
- * TpIntSetIter iter = TP_INTSET_INIT (intset);
+ * TpIntsetIter iter = TP_INTSET_INIT (intset);
  * while (tp_intset_iter_next (&amp;iter))
  * {
  *   printf ("%u is in the intset\n", iter.element);
  * }
  * </programlisting></informalexample>
  *
- * Since 0.11.6, consider using #TpIntSetFastIter if iteration in
+ * Since 0.11.6, consider using #TpIntsetFastIter if iteration in
  * numerical order is not required.
  *
  * Returns: %TRUE if (@iter->element) has been advanced
  */
 gboolean
-tp_intset_iter_next (TpIntSetIter *iter)
+tp_intset_iter_next (TpIntsetIter *iter)
 {
   g_return_val_if_fail (iter != NULL, FALSE);
   g_return_val_if_fail (iter->set != NULL, FALSE);
@@ -816,15 +857,18 @@ tp_intset_iter_next (TpIntSetIter *iter)
 }
 
 /**
- * TpIntSetFastIter:
+ * TpIntsetFastIter:
  *
  * An opaque structure representing iteration in undefined order over a set of
  * integers. Must be initialized with tp_intset_fast_iter_init().
  *
+ * Before 0.11.16, this type was called <type>TpIntSetFastIter</type>,
+ * which is now a backwards compatibility typedef.
+ *
  * Usage is similar to #GHashTableIter:
  *
  * <informalexample><programlisting>
- * TpIntSetFastIter iter;
+ * TpIntsetFastIter iter;
  * guint element;
  *
  * tp_intset_fast_iter_init (&amp;iter, intset);
@@ -845,7 +889,7 @@ typedef struct {
     gsize bitfield;
 } RealFastIter;
 
-G_STATIC_ASSERT (sizeof (TpIntSetFastIter) >= sizeof (RealFastIter));
+G_STATIC_ASSERT (sizeof (TpIntsetFastIter) >= sizeof (RealFastIter));
 
 /**
  * tp_intset_fast_iter_init:
@@ -858,8 +902,8 @@ G_STATIC_ASSERT (sizeof (TpIntSetFastIter) >= sizeof (RealFastIter));
  * Since: 0.11.6
  */
 void
-tp_intset_fast_iter_init (TpIntSetFastIter *iter,
-    const TpIntSet *set)
+tp_intset_fast_iter_init (TpIntsetFastIter *iter,
+    const TpIntset *set)
 {
   RealFastIter *real = (RealFastIter *) iter;
   g_return_if_fail (set != NULL);
@@ -884,7 +928,7 @@ tp_intset_fast_iter_init (TpIntSetFastIter *iter,
  * Since: 0.11.6
  */
 gboolean
-tp_intset_fast_iter_next (TpIntSetFastIter *iter,
+tp_intset_fast_iter_next (TpIntsetFastIter *iter,
     guint *output)
 {
   RealFastIter *real = (RealFastIter *) iter;
