@@ -90,7 +90,8 @@ tp_dbus_errors_quark (void)
  * @TP_DBUS_ERROR_CANCELLED: Raised from calls that re-enter the main
  *  loop (*_run_*) if they are cancelled
  * @TP_DBUS_ERROR_INCONSISTENT: Raised if information received from a remote
- *  object is inconsistent or otherwise obviously wrong (added in 0.7.17)
+ *  object is inconsistent or otherwise obviously wrong (added in 0.7.17).
+ *  See also %TP_ERROR_CONFUSED.
  *
  * #GError codes for use with the %TP_DBUS_ERRORS domain.
  *
@@ -342,9 +343,7 @@ struct _TpProxyPrivate {
     gboolean dispose_has_run;
 };
 
-G_DEFINE_TYPE (TpProxy,
-    tp_proxy,
-    G_TYPE_OBJECT);
+G_DEFINE_TYPE (TpProxy, tp_proxy, G_TYPE_OBJECT)
 
 enum
 {
@@ -1575,7 +1574,7 @@ _tp_proxy_is_preparing (gpointer self,
  * |[
  * TpChannel *channel = ...;
  *
- * tp_proxy_prepare_async (channel, NULL, NULL, callback, user_data);
+ * tp_proxy_prepare_async (channel, NULL, callback, user_data);
  * ]|
  *
  * is equivalent to
