@@ -24,13 +24,13 @@
  * @short_description: a subclass of #TpBaseClient implementing
  * a simple Handler
  *
- * This class makes it easier to write #TpSvcClient implementing the
- * TpSvcClientHandler interface.
+ * This class makes it easier to construct a #TpSvcClient implementing the
+ * #TpSvcClientHandler interface.
  *
  * A typical simple handler would look liks this:
  * |[
  * static void
- * my_handle_channels (TpSimpleHandler *self,
+ * my_handle_channels (TpSimpleHandler *handler,
  *    TpAccount *account,
  *    TpConnection *connection,
  *    GList *channels,
@@ -79,7 +79,7 @@
 
 /**
  * TpSimpleHandlerHandleChannelsImpl:
- * @self: a #TpSimpleHandler instance
+ * @handler: a #TpSimpleHandler instance
  * @account: a #TpAccount having %TP_ACCOUNT_FEATURE_CORE prepared if possible
  * @connection: a #TpConnection having %TP_CONNECTION_FEATURE_CORE prepared
  * if possible
@@ -264,7 +264,7 @@ tp_simple_handler_class_init (TpSimpleHandlerClass *cls)
   /**
    * TpSimpleHandler:callback:
    *
-   * The TpSimpleHandlerHandleChannelsImpl callback implementing the
+   * The #TpSimpleHandlerHandleChannelsImpl callback implementing the
    * HandleChannels D-Bus method.
    *
    * This property can't be %NULL.
@@ -281,8 +281,7 @@ tp_simple_handler_class_init (TpSimpleHandlerClass *cls)
   /**
    * TpSimpleHandler:user-data:
    *
-   * The user-data pointer passed to the callback implementing the
-   * HandleChannels D-Bus method.
+   * The user-data pointer passed to #TpSimpleHandler:callback.
    *
    * Since: 0.11.6
    */
@@ -295,8 +294,8 @@ tp_simple_handler_class_init (TpSimpleHandlerClass *cls)
   /**
    * TpSimpleHandler:destroy:
    *
-   * The #GDestroyNotify function called to free the user-data pointer when
-   * the #TpSimpleHandler is destroyed.
+   * The #GDestroyNotify function called to free #TpSimpleHandler:user-data
+   * when the #TpSimpleHandler is destroyed.
    *
    * Since: 0.11.6
    */
@@ -314,13 +313,13 @@ tp_simple_handler_class_init (TpSimpleHandlerClass *cls)
  * @dbus: a #TpDBusDaemon object, may not be %NULL
  * @bypass_approval: the value of the Handler.BypassApproval D-Bus property
  * (see tp_base_client_set_handler_bypass_approval() for details)
- * @requests: if this handler implement Requests (see
+ * @requests: whether this handler should implement Requests (see
  * tp_base_client_set_handler_request_notification() for details)
- * @name: the name of the Handler (see #TpBaseClient:name: for details)
- * @unique: the value of the TpBaseClient:uniquify-name: property
+ * @name: the name of the Handler (see #TpBaseClient:name for details)
+ * @uniquify: the value of the #TpBaseClient:uniquify-name property
  * @callback: the function called when HandleChannels is called
  * @user_data: arbitrary user-supplied data passed to @callback
- * @destroy: called with the user_data as argument, when the #TpSimpleHandler
+ * @destroy: called with @user_data as its argument when the #TpSimpleHandler
  * is destroyed
  *
  * Convenient function to create a new #TpSimpleHandler instance.
@@ -338,7 +337,7 @@ tp_simple_handler_new (TpDBusDaemon *dbus,
     gboolean bypass_approval,
     gboolean requests,
     const gchar *name,
-    gboolean unique,
+    gboolean uniquify,
     TpSimpleHandlerHandleChannelsImpl callback,
     gpointer user_data,
     GDestroyNotify destroy)
@@ -348,7 +347,7 @@ tp_simple_handler_new (TpDBusDaemon *dbus,
       "bypass-approval", bypass_approval,
       "requests", requests,
       "name", name,
-      "uniquify-name", unique,
+      "uniquify-name", uniquify,
       "callback", callback,
       "user-data", user_data,
       "destroy", destroy,
@@ -360,13 +359,13 @@ tp_simple_handler_new (TpDBusDaemon *dbus,
  * @account_manager: an account manager, which may not be %NULL
  * @bypass_approval: the value of the Handler.BypassApproval D-Bus property
  * (see tp_base_client_set_handler_bypass_approval() for details)
- * @requests: if this handler implement Requests (see
+ * @requests: whether this handler should implement Requests (see
  * tp_base_client_set_handler_request_notification() for details)
- * @name: the name of the Handler (see #TpBaseClient:name: for details)
- * @unique: the value of the TpBaseClient:uniquify-name: property
+ * @name: the name of the Handler (see #TpBaseClient:name for details)
+ * @uniquify: the value of the #TpBaseClient:uniquify-name property
  * @callback: the function called when HandleChannels is called
  * @user_data: arbitrary user-supplied data passed to @callback
- * @destroy: called with the user_data as argument, when the #TpSimpleHandler
+ * @destroy: called with @user_data as its argument when the #TpSimpleHandler
  * is destroyed
  *
  * Convenient function to create a new #TpSimpleHandler instance with a
@@ -384,7 +383,7 @@ tp_simple_handler_new_with_am (TpAccountManager *account_manager,
     gboolean bypass_approval,
     gboolean requests,
     const gchar *name,
-    gboolean unique,
+    gboolean uniquify,
     TpSimpleHandlerHandleChannelsImpl callback,
     gpointer user_data,
     GDestroyNotify destroy)
@@ -394,7 +393,7 @@ tp_simple_handler_new_with_am (TpAccountManager *account_manager,
       "bypass-approval", bypass_approval,
       "requests", requests,
       "name", name,
-      "uniquify-name", unique,
+      "uniquify-name", uniquify,
       "callback", callback,
       "user-data", user_data,
       "destroy", destroy,
