@@ -29,16 +29,6 @@
 #include "tests/lib/myassert.h"
 #include "tests/lib/util.h"
 
-typedef struct {
-  TpBaseConnection *base_connection;
-  TpBaseConnection *legacy_base_connection;
-  TpBaseConnection *no_requests_base_connection;
-  TpTestsContactsConnection *service_conn;
-  TpConnection *client_conn;
-  TpConnection *legacy_client_conn;
-  TpConnection *no_requests_client_conn;
-} Fixture;
-
 typedef struct {
     GMainLoop *loop;
     GError *error /* initialized to 0 */;
@@ -48,6 +38,18 @@ typedef struct {
     GHashTable *bad_ids;
 } Result;
 
+typedef struct {
+  Result result;
+  TpBaseConnection *base_connection;
+  TpBaseConnection *legacy_base_connection;
+  TpBaseConnection *no_requests_base_connection;
+  TpTestsContactsConnection *service_conn;
+  TpHandleRepoIface *service_repo;
+  TpConnection *client_conn;
+  TpConnection *legacy_client_conn;
+  TpConnection *no_requests_client_conn;
+} Fixture;
+
 static void
 by_handle_cb (TpConnection *connection,
               guint n_contacts,
@@ -126,32 +128,13 @@ finish (gpointer r)
 static void
 reset_result (Result *result)
 {
-  g_clear_error (&(result->error));
-
+  tp_clear_pointer (&result->invalid, g_array_unref);
   if (result->contacts != NULL)
-    {
-      g_ptr_array_foreach (result->contacts, (GFunc) g_object_unref, NULL);
-      g_ptr_array_unref (result->contacts);
-      result->contacts = NULL;
-    }
-
-  if (result->invalid)
-    {
-      g_array_unref (result->invalid);
-      result->invalid = NULL;
-    }
-
-  if (result->good_ids)
-    {
-      g_strfreev (result->good_ids);
-      result->good_ids = NULL;
-    }
-
-  if (result->bad_ids)
-    {
-      g_hash_table_unref (result->bad_ids);
-      result->bad_ids = NULL;
-    }
+    g_ptr_array_foreach (result->contacts, (GFunc) g_object_unref, NULL);
+  tp_clear_pointer (&result->contacts, g_ptr_array_unref);
+  tp_clear_pointer (&result->good_ids, g_strfreev);
+  tp_clear_pointer (&result->bad_ids, g_hash_table_unref);
+  g_clear_error (&result->error);
 }
 
 static void
@@ -802,16 +785,6 @@ test_by_handle (Fixture *f,
   /* wait for ReleaseHandles to run */
   tp_tests_proxy_run_until_dbus_queue_processed (client_conn);
 
-  /* unref all the handles we created service-side */
-  tp_handle_unref (service_repo, handles[0]);
-  MYASSERT (!tp_handle_is_valid (service_repo, handles[0], NULL), "");
-  tp_handle_unref (service_repo, handles[1]);
-  MYASSERT (!tp_handle_is_valid (service_repo, handles[1], NULL), "");
-  tp_handle_unref (service_repo, handles[2]);
-  MYASSERT (!tp_handle_is_valid (service_repo, handles[2], NULL), "");
-  tp_handle_unref (service_repo, handles[3]);
-  MYASSERT (!tp_handle_is_valid (service_repo, handles[3], NULL), "");
-
   /* remaining cleanup */
   g_assert (result.error == NULL);
   reset_result (&result);
@@ -1049,7 +1022,6 @@ test_no_features (Fixture *f,
       g_object_unref (contacts[i]);
       tp_tests_proxy_run_until_dbus_queue_processed (client_conn);
       tp_handle_unref (service_repo, handles[i]);
-      MYASSERT (!tp_handle_is_valid (service_repo, handles[i], NULL), "");
     }
 
   /* remaining cleanup */
@@ -1277,7 +1249,7 @@ test_upgrade (Fixture *f,
 
   tp_connection_upgrade_contacts (client_conn,
       3, contacts,
-      sizeof (features) / sizeof (features[0]), features,
+      G_N_ELEMENTS (features), features,
       upgrade_cb,
       &result, finish, NULL);
 
@@ -1345,7 +1317,6 @@ test_upgrade (Fixture *f,
       g_object_unref (contacts[i]);
       tp_tests_proxy_run_until_dbus_queue_processed (client_conn);
       tp_handle_unref (service_repo, handles[i]);
-      MYASSERT (!tp_handle_is_valid (service_repo, handles[i], NULL), "");
     }
 
   /* remaining cleanup */
@@ -1525,7 +1496,7 @@ test_features (Fixture *f,
 
   tp_connection_get_contacts_by_handle (client_conn,
       3, handles,
-      sizeof (features) / sizeof (features[0]), features,
+      G_N_ELEMENTS (features), features,
       by_handle_cb,
       &result, finish, NULL);
 
@@ -1716,7 +1687,6 @@ test_features (Fixture *f,
       g_object_unref (contacts[i]);
       tp_tests_proxy_run_until_dbus_queue_processed (client_conn);
       tp_handle_unref (service_repo, handles[i]);
-      MYASSERT (!tp_handle_is_valid (service_repo, handles[i], NULL), "");
     }
 
   /* remaining cleanup */
@@ -1925,7 +1895,7 @@ test_capabilities_without_contact_caps (Fixture *f,
 
   tp_connection_get_contacts_by_handle (client_conn,
       3, handles,
-      sizeof (features) / sizeof (features[0]), features,
+      G_N_ELEMENTS (features), features,
       by_handle_cb,
       &result, finish, NULL);
 
@@ -1988,7 +1958,7 @@ test_prepare_contact_caps_without_request (Fixture *f,
 
   tp_connection_get_contacts_by_handle (client_conn,
       3, handles,
-      sizeof (features) / sizeof (features[0]), features,
+      G_N_ELEMENTS (features), features,
       by_handle_cb,
       &result, finish, NULL);
 
@@ -2029,6 +1999,248 @@ test_prepare_contact_caps_without_request (Fixture *f,
   g_main_loop_unref (result.loop);
 }
 
+static void
+test_dup_if_possible (Fixture *f,
+    gconstpointer unused G_GNUC_UNUSED)
+{
+  TpHandle alice_handle, bob_handle;
+  TpContact *alice;
+  TpContact *contact;
+
+  alice_handle = tp_handle_ensure (f->service_repo, "alice", NULL, NULL);
+  g_assert_cmpuint (alice_handle, !=, 0);
+  bob_handle = tp_handle_ensure (f->service_repo, "bob", NULL, NULL);
+  g_assert_cmpuint (bob_handle, !=, 0);
+
+  tp_connection_get_contacts_by_handle (f->client_conn,
+      1, &alice_handle,
+      0, NULL,
+      by_handle_cb,
+      &f->result, finish, NULL);
+  g_main_loop_run (f->result.loop);
+  g_assert_cmpuint (f->result.contacts->len, ==, 1);
+  g_assert_cmpuint (f->result.invalid->len, ==, 0);
+  g_assert_no_error (f->result.error);
+
+  g_assert (g_ptr_array_index (f->result.contacts, 0) != NULL);
+  alice = g_object_ref (g_ptr_array_index (f->result.contacts, 0));
+  g_assert_cmpuint (tp_contact_get_handle (alice), ==, alice_handle);
+  g_assert_cmpstr (tp_contact_get_identifier (alice), ==, "alice");
+
+  reset_result (&f->result);
+
+  /* we already have a cached TpContact for Alice, so we can get another
+   * copy of it synchronously */
+
+  contact = tp_connection_dup_contact_if_possible (f->client_conn,
+      alice_handle, "alice");
+  g_assert (contact == alice);
+  g_object_unref (contact);
+
+  contact = tp_connection_dup_contact_if_possible (f->client_conn,
+      alice_handle, NULL);
+  g_assert (contact == alice);
+  g_object_unref (contact);
+
+  /* because this connection has immortal handles, we can reliably get a
+   * contact for Bob synchronously, but only if we supply his identifier */
+
+  contact = tp_connection_dup_contact_if_possible (f->client_conn,
+      bob_handle, NULL);
+  g_assert (contact == NULL);
+
+  contact = tp_connection_dup_contact_if_possible (f->client_conn,
+      bob_handle, "bob");
+  g_assert (contact != alice);
+  g_assert_cmpstr (tp_contact_get_identifier (contact), ==, "bob");
+  g_assert_cmpuint (tp_contact_get_handle (contact), ==, bob_handle);
+}
+
+typedef struct
+{
+  TpSubscriptionState subscribe;
+  TpSubscriptionState publish;
+  const gchar *publish_request;
+
+  GMainLoop *loop;
+} SubscriptionStates;
+
+static void
+assert_subscription_states (TpContact *contact,
+    SubscriptionStates *states)
+{
+  g_assert_cmpint (tp_contact_get_subscribe_state (contact), ==, states->subscribe);
+  g_assert_cmpint (tp_contact_get_publish_state (contact), ==, states->publish);
+  g_assert_cmpstr (tp_contact_get_publish_request (contact), ==, states->publish_request);
+}
+
+static void
+subscription_states_changed_cb (TpContact *contact,
+    TpSubscriptionState subscribe,
+    TpSubscriptionState publish,
+    const gchar *publish_request,
+    SubscriptionStates *states)
+{
+  assert_subscription_states (contact, states);
+  g_main_loop_quit (states->loop);
+}
+
+static void
+test_subscription_states (Fixture *f,
+    gconstpointer unused G_GNUC_UNUSED)
+{
+  TpHandle alice_handle;
+  TpContact *alice;
+  TestContactListManager *manager;
+  TpContactFeature features[] = { TP_CONTACT_FEATURE_SUBSCRIPTION_STATES };
+  SubscriptionStates states = { TP_SUBSCRIPTION_STATE_NO,
+      TP_SUBSCRIPTION_STATE_NO, "", f->result.loop };
+
+  manager = tp_tests_contacts_connection_get_contact_list_manager (
+      f->service_conn);
+
+  alice_handle = tp_handle_ensure (f->service_repo, "alice", NULL, NULL);
+  g_assert_cmpuint (alice_handle, !=, 0);
+
+  tp_connection_get_contacts_by_handle (f->client_conn,
+      1, &alice_handle,
+      G_N_ELEMENTS (features), features,
+      by_handle_cb,
+      &f->result, finish, NULL);
+  g_main_loop_run (f->result.loop);
+  g_assert_cmpuint (f->result.contacts->len, ==, 1);
+  g_assert_cmpuint (f->result.invalid->len, ==, 0);
+  g_assert_no_error (f->result.error);
+
+  g_assert (g_ptr_array_index (f->result.contacts, 0) != NULL);
+  alice = g_object_ref (g_ptr_array_index (f->result.contacts, 0));
+  g_assert_cmpuint (tp_contact_get_handle (alice), ==, alice_handle);
+  g_assert_cmpstr (tp_contact_get_identifier (alice), ==, "alice");
+  assert_subscription_states (alice, &states);
+
+  reset_result (&f->result);
+
+  g_signal_connect (alice, "subscription-states-changed",
+      G_CALLBACK (subscription_states_changed_cb), &states);
+
+  /* Request subscription */
+  test_contact_list_manager_request_subscription (manager, 1, &alice_handle, "");
+  states.subscribe = TP_SUBSCRIPTION_STATE_ASK;
+  g_main_loop_run (states.loop);
+
+  /* Request again must re-emit the signal. Saying please this time will make
+   * the request accepted and will ask for publish. */
+  test_contact_list_manager_request_subscription (manager, 1, &alice_handle, "please");
+  g_main_loop_run (states.loop);
+  states.subscribe = TP_SUBSCRIPTION_STATE_YES;
+  states.publish = TP_SUBSCRIPTION_STATE_ASK;
+  states.publish_request = "automatic publish request";
+  g_main_loop_run (states.loop);
+
+  /* Remove the contact */
+  test_contact_list_manager_remove (manager, 1, &alice_handle);
+  states.subscribe = TP_SUBSCRIPTION_STATE_NO;
+  states.publish = TP_SUBSCRIPTION_STATE_NO;
+  states.publish_request = "";
+  g_main_loop_run (states.loop);
+
+  g_object_unref (alice);
+}
+
+typedef struct
+{
+  GPtrArray *groups;
+  GMainLoop *loop;
+} ContactGroups;
+
+static void
+assert_contact_groups (TpContact *contact,
+    ContactGroups *data)
+{
+  const gchar * const *groups = tp_contact_get_contact_groups (contact);
+  guint i;
+
+  g_assert (groups != NULL);
+  g_assert_cmpuint (g_strv_length ((GStrv) groups), ==, data->groups->len);
+
+  for (i = 0; i < data->groups->len; i++)
+    g_assert (tp_strv_contains (groups, g_ptr_array_index (data->groups, i)));
+}
+
+static void
+contact_groups_changed_cb (TpContact *contact,
+    GStrv added,
+    GStrv removed,
+    ContactGroups *data)
+{
+  assert_contact_groups (contact, data);
+  g_main_loop_quit (data->loop);
+}
+
+static void
+test_contact_groups (Fixture *f,
+    gconstpointer unused G_GNUC_UNUSED)
+{
+  TpHandle alice_handle;
+  TpContact *alice;
+  TestContactListManager *manager;
+  TpContactFeature features[] = { TP_CONTACT_FEATURE_CONTACT_GROUPS };
+  ContactGroups data;
+
+  data.groups = g_ptr_array_new ();
+  data.loop = f->result.loop;
+
+  manager = tp_tests_contacts_connection_get_contact_list_manager (
+      f->service_conn);
+
+  alice_handle = tp_handle_ensure (f->service_repo, "alice", NULL, NULL);
+  g_assert_cmpuint (alice_handle, !=, 0);
+
+  tp_connection_get_contacts_by_handle (f->client_conn,
+      1, &alice_handle,
+      G_N_ELEMENTS (features), features,
+      by_handle_cb,
+      &f->result, finish, NULL);
+  g_main_loop_run (f->result.loop);
+  g_assert_cmpuint (f->result.contacts->len, ==, 1);
+  g_assert_cmpuint (f->result.invalid->len, ==, 0);
+  g_assert_no_error (f->result.error);
+
+  g_assert (g_ptr_array_index (f->result.contacts, 0) != NULL);
+  alice = g_object_ref (g_ptr_array_index (f->result.contacts, 0));
+  g_assert_cmpuint (tp_contact_get_handle (alice), ==, alice_handle);
+  g_assert_cmpstr (tp_contact_get_identifier (alice), ==, "alice");
+  assert_contact_groups (alice, &data);
+
+  reset_result (&f->result);
+
+  g_signal_connect (alice, "contact-groups-changed",
+      G_CALLBACK (contact_groups_changed_cb), &data);
+
+  g_ptr_array_add (data.groups, "group1");
+  test_contact_list_manager_add_to_group (manager, "group1", alice_handle);
+  g_main_loop_run (data.loop);
+
+  g_ptr_array_add (data.groups, "group2");
+  test_contact_list_manager_add_to_group (manager, "group2", alice_handle);
+  g_main_loop_run (data.loop);
+
+  g_ptr_array_remove_index_fast (data.groups, 0);
+  test_contact_list_manager_remove_from_group (manager, "group1", alice_handle);
+  g_main_loop_run (data.loop);
+
+  g_ptr_array_set_size (data.groups, 0);
+  g_ptr_array_add (data.groups, "group1");
+  g_ptr_array_add (data.groups, "group2");
+  g_ptr_array_add (data.groups, "group3");
+  tp_contact_set_contact_groups_async (alice, data.groups->len,
+      (const gchar * const *) data.groups->pdata, NULL, NULL);
+  g_main_loop_run (data.loop);
+
+  g_ptr_array_unref (data.groups);
+  g_object_unref (alice);
+}
+
 static void
 setup (Fixture *f,
     gconstpointer unused G_GNUC_UNUSED)
@@ -2045,6 +2257,10 @@ setup (Fixture *f,
   tp_tests_create_and_connect_conn (TP_TESTS_TYPE_NO_REQUESTS_CONNECTION,
       "me3@test.com", &f->no_requests_base_connection,
       &f->no_requests_client_conn);
+
+  f->service_repo = tp_base_connection_get_handles (f->base_connection,
+      TP_HANDLE_TYPE_CONTACT);
+  f->result.loop = g_main_loop_new (NULL, FALSE);
 }
 
 static void
@@ -2062,6 +2278,7 @@ teardown (Fixture *f,
     }
 
   tp_clear_object (&f->client_conn);
+  f->service_repo = NULL;
   tp_clear_object (&f->service_conn);
   tp_clear_object (&f->base_connection);
 
@@ -2086,6 +2303,8 @@ teardown (Fixture *f,
 
   tp_clear_object (&f->no_requests_client_conn);
   tp_clear_object (&f->no_requests_base_connection);
+  reset_result (&f->result);
+  tp_clear_pointer (&f->result.loop, g_main_loop_unref);
 }
 
 int
@@ -2094,6 +2313,7 @@ main (int argc,
 {
   g_type_init ();
   tp_debug_set_flags ("all");
+  tp_tests_abort_after (10);
   g_set_prgname ("contacts");
   g_test_init (&argc, &argv, NULL);
   g_test_bug_base ("http://bugs.freedesktop.org/show_bug.cgi?id=");
@@ -2111,6 +2331,10 @@ main (int argc,
   ADD (avatar_requirements);
   ADD (avatar_data);
   ADD (contact_info);
+  ADD (dup_if_possible);
+  ADD (subscription_states);
+  ADD (contact_groups);
+
   /* test if TpContact fallbacks to connection's capabilities if
    * ContactCapabilities is not implemented. */
   ADD (capabilities_without_contact_caps);
